"""Tests for the views of the ``server_guardian`` app."""
import os
import subprocess

from django.core.urlresolvers import reverse
from django.test import TestCase

from django_libs.tests.mixins import ViewRequestFactoryTestMixin
from mixer.backend.django import mixer
from mock import MagicMock

from .. import views


class GuardianDashboardViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``GuardianDashboardView`` view class."""
    view_class = views.GuardianDashboardView

    def setUp(self):
        self.user = mixer.blend('auth.User')
        self.admin = mixer.blend('auth.User', is_superuser=True)

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        self.is_callable(user=self.admin)
        self.redirects(user=self.user, to='{0}?next={1}'.format(
            self.get_login_url(), self.get_url()))


class GuardianReloadViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``GuardianReloadView`` view class."""
    view_class = views.GuardianReloadView

    def setUp(self):
        self.user = mixer.blend('auth.User')
        self.admin = mixer.blend('auth.User', is_superuser=True)
        self.popen = subprocess.Popen
        self.exists = os.path.exists

    def tearDown(self):
        subprocess.Popen = self.popen
        os.path.exists = self.exists

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        self.redirects(user=self.user, to='{0}?next={1}'.format(
            self.get_login_url(), self.get_url()))
        self.redirects(
            user=self.admin, to=reverse('server_guardian_dashboard'))
        self.is_callable(user=self.admin, ajax=True)

        os.path.exists = MagicMock(return_value=True)
        self.is_postable(user=self.admin, data={}, ajax=True)

        os.path.exists = MagicMock(return_value=False)
        subprocess.Popen = MagicMock()
        self.is_postable(user=self.admin, data={}, ajax=True)

        def exception():
            raise Exception('Foobar')
        subprocess.Popen = MagicMock(side_effect=exception)
        self.is_postable(user=self.admin, data={}, ajax=True)



"""Settings that need to be set in order to run the tests."""
import os
import logging

# prevent output of verbose DB gibberish
logging.getLogger("django").setLevel(logging.WARN)

DEBUG = True

SITE_ID = 1

APP_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '..'))

PROJECT_ROOT = os.path.join(APP_ROOT, '..')

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

ROOT_URLCONF = 'server_guardian.tests.urls'

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(APP_ROOT, '../app_static')
MEDIA_ROOT = os.path.join(APP_ROOT, '../app_media')
STATICFILES_DIRS = (
    os.path.join(APP_ROOT, 'static'),
)

TEMPLATE_DIRS = (
    os.path.join(APP_ROOT, 'tests/test_app/templates'),
)

EXTERNAL_APPS = [
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
    'django_nose',
]

INTERNAL_APPS = [
    'server_guardian',
    'server_guardian_api',
    'server_guardian.tests.test_app',
]

SERVER_GUARDIAN_SECURITY_TOKEN = 'foobar'

LOGIN_URL = 'admin/login/'

LOCKFILE_FOLDER = os.environ.get('HOME')

INSTALLED_APPS = EXTERNAL_APPS + INTERNAL_APPS

MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

COVERAGE_REPORT_HTML_OUTPUT_DIR = os.path.join(
    os.path.join(APP_ROOT, 'tests/coverage'))
COVERAGE_MODULE_EXCLUDES = [
    'tests$', 'settings$', 'urls$', 'locale$',
    'migrations', 'fixtures', 'admin$', 'django_extensions',
]
COVERAGE_MODULE_EXCLUDES += EXTERNAL_APPS

SECRET_KEY = 'foobar'

FROM_EMAIL = 'foobar@example.com'
EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'

ADMINS = (
    ('Fooman Barster', 'fooman.barster@example.com'),
)



#!/usr/bin/env python
"""
This script sets up the test runner, that is called within runtests.py.

"""
import sys


import django
from django.conf import settings

import test_settings

if not settings.configured:
    settings.configure(**test_settings.__dict__)


from django_nose import NoseTestSuiteRunner


def runtests(*test_args):
    if django.VERSION >= (1, 7):
        django.setup()

    failures = NoseTestSuiteRunner(verbosity=2, interactive=True).run_tests(
        test_args)

    sys.exit(failures)


if __name__ == '__main__':
    runtests(*sys.argv[1:])



"""Models for the server_guardian app."""
import json

from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from compat import python_2_unicode_compatible

from . import constants


@python_2_unicode_compatible
class Server(models.Model):
    name = models.CharField(
        max_length=256,
        blank=True,
        verbose_name=_('server name'),
    )
    url = models.URLField(
        verbose_name=_('API URL'),
        help_text='The URL, that the guardian API is available under.'
    )
    token = models.CharField(
        max_length=256,
        verbose_name=_('token'),
        help_text=(
            'Add this to your client server settings as'
            ' "SERVER_GUARDIAN_SECURITY_TOKEN".'
        ),
    )
    response_body = models.TextField(
        verbose_name=_('server response'),
        blank=True,
    )
    status_code = models.PositiveIntegerField(
        verbose_name=_('server response status code'),
        blank=True, null=True,
    )
    last_updated = models.DateTimeField(
        verbose_name=_('last updated'),
        blank=True, null=True,
    )

    def __unicode__(self):
        return self.__str__()

    def __str__(self):  # pragma: no cover
        if self.name:
            return self.name
        else:
            return self.url

    def get_absolute_url(self):
        return reverse('server_guardian_dashboard')

    def get_parsed_response(self):
        try:
            return json.loads(self.response_body)
        except ValueError:
            return constants.HTML_STATUS_FALLBACK

    def has_errors(self):
        for metric in self.get_parsed_response():
            if metric['status'] in constants.ERROR_STATUS:
                return True
        return False



#!/usr/bin/env python
"""
This script is a trick to setup a fake Django environment, since this reusable
app will be developed and tested outside any specific Django project.

Via ``settings.configure`` you will be able to set all necessary settings
for your app and run the tests as if you were calling ``./manage.py test``.

"""
import re

import django
from django.conf import settings

from fabric.api import abort, lcd, local
from fabric.colors import green, red

import test_settings

if not settings.configured:
    settings.configure(**test_settings.__dict__)

if __name__ == '__main__':
    if django.VERSION >= (1, 7):
        django.setup()
    local('coverage run testrunner.py')
    local('coverage html -d "{0}"'.format(
        settings.COVERAGE_REPORT_HTML_OUTPUT_DIR))

    local(
        'flake8 --ignore=E126 --ignore=W391 --statistics'
        ' --exclude=submodules,migrations,build .')

    with lcd(settings.COVERAGE_REPORT_HTML_OUTPUT_DIR):
        total_line = local('grep -n pc_cov index.html', capture=True)
        percentage = float(re.findall(r'(\d+)%', total_line)[-1])
    if percentage < 100:
        abort(red('Coverage is {0}%'.format(percentage)))
    print(green('Coverage is {0}%'.format(percentage)))
